# Import required modules
from __future__ import unicode_literals
from concurrent.futures import ThreadPoolExecutor
import os
import pickle
import textwrap
import yt_dlp

# Change working directory
os.chdir(os.path.dirname(os.path.abspath(__file__)))

# Load contents of options file
with open('options', 'rb') as load_opts_file:
    opts_list = pickle.load(load_opts_file)

# Menu layout
def disp_menu():
    print(24 * '-', 'MENU', 24 * '-')
    print('1. Download videos (input URLs)')
    print('2. Download videos (load URLs from file)')
    print('3. Download subtitles, true/false:', opts_list[0])
    print('4. Change subtitle language, current language/s:', *opts_list[1])
    print('5. Change video download location, current location:', opts_list[2])
    print('6. Exit')
    print(54 * '-')

# Save program options to file
def save_opts():
    with open('options', 'wb') as save_opts_file:
        pickle.dump(opts_list, save_opts_file)

# Set downloader/thread options + start threads
def down_thread_opts():
    vid_down_opts = {
        'geo_bypass': True,
        'outtmpl': os.path.join(opts_list[2], os.path.normpath('%(uploader|)s/%(title)s.%(ext)s')),
        'nooverwrites': True,
        'restrictfilenames': True,
        'subtitleslangs': opts_list[1],
        'writesubtitles': opts_list[0]
    }
    if len(vid_url_inp) < 3:
        with yt_dlp.YoutubeDL(vid_down_opts) as ydl:
            ydl.download(vid_url_inp)
    else:
        with ThreadPoolExecutor(max_workers = 3) as executor:
            executor.submit(down_vids, vid_url_inp[::3], vid_down_opts)
            executor.submit(down_vids_a, vid_url_inp[1::3], vid_down_opts)
            executor.submit(down_vids_b, vid_url_inp[2::3], vid_down_opts)

# Execute yt-dlp with desired options
def down_vids(vid_urls, ydl_opts):
    with yt_dlp.YoutubeDL(ydl_opts) as ydl:
        ydl.download(vid_urls)
def down_vids_a(vid_urls_a, ydl_opts_a):
    with yt_dlp.YoutubeDL(ydl_opts_a) as ydl:
        ydl.download(vid_urls_a)
def down_vids_b(vid_urls_b, ydl_opts_b):
    with yt_dlp.YoutubeDL(ydl_opts_b) as ydl:
        ydl.download(vid_urls_b)

# Loop until user chooses to exit
loop = True

# Menu options
while loop:
    disp_menu()
    menu_opt = input('Enter a number then press enter: ')
    if menu_opt == '1':
        vid_url_inp = input(textwrap.dedent('''\
        Enter a video/playlist URL then press enter,
        (multiple URLs can be entered separated by a space): ''')).split()
        if __name__ == "__main__":
            down_thread_opts()
        print('Download Complete.\n')
    elif menu_opt == '2':
        with open('video_urls.txt', 'r') as vid_url_file:
            loaded_urls = vid_url_file.readlines()
        vid_url_inp = ' '.join(loaded_urls).split()
        print('Successfully loaded URLs from file.')
        if __name__ == "__main__":
            down_thread_opts()
        print('Download Complete.\n')
    elif menu_opt == '3':
        sub_down_inp = input('Download subtitles? Enter y or n then press enter: ')
        if sub_down_inp == 'y':
            opts_list[0] = True
        else:
            opts_list[0] = False
        save_opts()
    elif menu_opt == '4':
        opts_list[1] = input(textwrap.dedent('''\
        Enter a language code then press enter,
        (multiple codes can be entered separated by a space): ''')).split()
        save_opts()
    elif menu_opt == '5':
        opts_list[2] = input('Enter your preferred video download location then press enter: ')
        save_opts()
    elif menu_opt == '6':
        print('Goodbye.')
        loop = False
    else:
        print('Please enter a valid number.')